@@ -1,5 +1,12 @@
 use crate::simd::*;
-use std::{cmp::Ordering, convert::TryFrom, error::Error, fmt, result};
+use std::{
+    cmp::Ordering,
+    convert::TryFrom,
+    error::Error,
+    fmt,
+    hash::{Hash, Hasher},
+    result,
+};
 
 #[derive(Clone, Copy, Debug, Eq, PartialEq)]
 pub enum IntervalErrorKind {
@@ -8,24 +15,18 @@ pub enum IntervalErrorKind {
 }
 
 #[derive(Clone, Debug, Eq, PartialEq)]
-pub struct IntervalError<T: fmt::Debug> {
+pub struct IntervalError {
     pub(crate) kind: IntervalErrorKind,
-    pub(crate) value: T,
 }
 
-impl<T: fmt::Debug> IntervalError<T> {
+impl IntervalError {
     /// Returns the type of the error.
     pub fn kind(&self) -> IntervalErrorKind {
         self.kind
     }
-
-    /// Returns a reasonable fallback value for the error.
-    pub fn value(self) -> T {
-        self.value
-    }
 }
 
-impl<T: fmt::Debug> fmt::Display for IntervalError<T> {
+impl fmt::Display for IntervalError {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match self.kind {
             IntervalErrorKind::PossiblyUndefinedOperation => {
@@ -36,12 +37,12 @@ impl<T: fmt::Debug> fmt::Display for IntervalError<T> {
     }
 }
 
-impl<T: fmt::Debug> Error for IntervalError<T> {}
+impl Error for IntervalError {}
 
-/// An alias for `Result` with [`IntervalError`].
-pub type Result<T> = result::Result<T, IntervalError<T>>;
+/// An alias for [`Result<T, E>`](`result::Result`) with [`E = IntervalError`](`IntervalError`).
+pub type Result<T> = result::Result<T, IntervalError>;
 
-/// An interval with `f64` bounds.
+/// An interval with [`f64`] bounds.
 ///
 /// It is sometimes referred to as a *bare* interval in contrast to a decorated interval ([`DecInterval`]).
 #[derive(Clone, Copy, Debug)]
@@ -95,8 +96,15 @@ impl PartialEq for Interval {
 
 impl Eq for Interval {}
 
+impl Hash for Interval {
+    fn hash<H: Hasher>(&self, state: &mut H) {
+        self.inf().to_bits().hash(state);
+        self.sup().to_bits().hash(state);
+    }
+}
+
 impl TryFrom<(f64, f64)> for Interval {
-    type Error = IntervalError<Self>;
+    type Error = IntervalError;
 
     fn try_from((a, b): (f64, f64)) -> Result<Self> {
         if a <= b && a != f64::INFINITY && b != f64::NEG_INFINITY {
@@ -104,14 +112,13 @@ impl TryFrom<(f64, f64)> for Interval {
         } else {
             Err(Self::Error {
                 kind: IntervalErrorKind::UndefinedOperation,
-                value: Self::EMPTY,
             })
         }
     }
 }
 
 /// The decoration of a [`DecInterval`].
-#[derive(Clone, Copy, Debug, Eq, PartialEq)]
+#[derive(Clone, Copy, Debug, Eq, Hash, PartialEq)]
 #[repr(u8)]
 pub enum Decoration {
     Ill = 0,
@@ -133,7 +140,16 @@ impl PartialOrd for Decoration {
     }
 }
 
-/// A decorated interval with `f64` bounds.
+/// A decorated interval with [`f64`] bounds.
+///
+/// Note that by definition, a NaI is not equal to itself:
+///
+/// ```
+/// use inari::*;
+/// assert_ne!(DecInterval::NAI, DecInterval::NAI);
+/// ```
+///
+/// For this reason, the traits [`Eq`] and [`Hash`] are not implemented for the type.
 #[derive(Clone, Copy, Debug)]
 #[repr(C)]
 pub struct DecInterval {
@@ -197,17 +213,14 @@ impl PartialEq for DecInterval {
     }
 }
 
-// `DecInterval` is not a model of `Eq` as NaI ≠ NaI.
-
 impl TryFrom<(f64, f64)> for DecInterval {
-    type Error = IntervalError<Self>;
+    type Error = IntervalError;
 
     fn try_from(x: (f64, f64)) -> Result<Self> {
         match Interval::try_from(x) {
             Ok(x) => Ok(Self::new(x)),
             _ => Err(Self::Error {
                 kind: IntervalErrorKind::UndefinedOperation,
-                value: Self::NAI,
             }),
         }
     }
@@ -233,22 +246,46 @@ macro_rules! interval {
     };
 }
 
-/// Creates an [`Interval`] from `f64` bounds or a bare interval literal.
+/// Creates an [`Interval`] from [`f64`] bounds or from a bare interval literal.
+///
+/// In case of a failure, it returns a [`Result<Interval>`] instead of an [`Interval`].
+///
+/// If the construction is invalid,
+/// an [`Err`] with [`IntervalErrorKind::UndefinedOperation`] is returned.
+/// If it cannot determine whether the construction is valid or not,
+/// [`IntervalErrorKind::PossiblyUndefinedOperation`] is returned.
+///
+/// - `interval!(a, b)`
+///
+///   Creates an interval $\[a, b\]$, where `a` and `b` are [`f64`] values.
+///
+///   The conditions $a ≤ b$, $a < +∞$ and $b > -∞$ must be held.
+///
+/// - `interval!(s)`
+///
+///   Creates an interval from a bare interval literal. `s` must be a string slice ([`&str`]).
+///
+/// - `interval!(s, exact)`
+///
+///   Creates an interval $𝒙$ from the bare interval literal obtained by `format!("{:x}", x)`.
+///   `s` must be a string slice ([`&str`]).
+///
+/// For creating a constant, the macro [`const_interval!`] should be preferred over this one.
 #[cfg(feature = "gmp")]
 #[macro_export]
 macro_rules! interval {
-    ($text:expr) => {{
+    ($s:expr) => {{
         use ::std::primitive::*;
         fn is_str(_: &str) {}
-        is_str($text);
-        $text.parse::<$crate::Interval>()
+        is_str($s);
+        $s.parse::<$crate::Interval>()
     }};
 
-    ($text:expr, exact) => {{
+    ($s:expr, exact) => {{
         use ::std::primitive::*;
         fn is_str(_: &str) {}
-        is_str($text);
-        $crate::Interval::_try_from_str_exact($text)
+        is_str($s);
+        $crate::Interval::_try_from_str_exact($s)
     }};
 
     ($a:expr, $b:expr) => {
@@ -276,15 +313,36 @@ macro_rules! dec_interval {
     };
 }
 
-/// Creates a [`DecInterval`] from `f64` bounds or a decorated interval literal.
+/// Creates a [`DecInterval`] from [`f64`] bounds or from a decorated interval literal.
+///
+/// In case of a failure, it returns a [`Result<DecInterval>`] instead of a [`DecInterval`].
+///
+/// If the construction is invalid,
+/// an [`Err`] with [`IntervalErrorKind::UndefinedOperation`] is returned.
+/// If it cannot determine whether the construction is valid or not,
+/// [`IntervalErrorKind::PossiblyUndefinedOperation`] is returned.
+///
+/// - `dec_interval!(a, b)`
+///
+///   Creates a decorated interval $\[a, b\]$ with the strongest decoration,
+///   where `a` and `b` are [`f64`] values.
+///
+///   The conditions $a ≤ b$, $a < +∞$ and $b > -∞$ must be held.
+///
+/// - `dec_interval!(s)`
+///
+///   Creates a decorated interval from a decorated interval literal.
+///   `s` must be a string slice ([`&str`]).
+///
+/// For creating a constant, the macro [`const_dec_interval!`] should be preferred over this one.
 #[cfg(feature = "gmp")]
 #[macro_export]
 macro_rules! dec_interval {
-    ($text:expr) => {{
+    ($s:expr) => {{
         use ::std::primitive::*;
         fn is_str(_: &str) {}
-        is_str($text);
-        $text.parse::<$crate::DecInterval>()
+        is_str($s);
+        $s.parse::<$crate::DecInterval>()
     }};
 
     ($a:expr, $b:expr) => {
@@ -292,9 +350,13 @@ macro_rules! dec_interval {
     };
 }
 
-/// Creates an [`Interval`] from `f64` bounds.
+/// Creates an [`Interval`] from [`f64`] bounds.
 ///
 /// It can be used in [constant expressions](https://doc.rust-lang.org/reference/const_eval.html#constant-expressions).
+///
+/// The usage is almost the same as the macro [`interval!(a, b)`](`interval!`),
+/// except that `const_interval!(a, b)` returns an [`Interval`]
+/// or results in a compilation error if the construction is invalid.
 #[macro_export]
 macro_rules! const_interval {
     ($a:expr, $b:expr) => {{
@@ -312,9 +374,13 @@ macro_rules! const_interval {
     }};
 }
 
-/// Creates a [`DecInterval`] from `f64` bounds.
+/// Creates a [`DecInterval`] from [`f64`] bounds.
 ///
 /// It can be used in [constant expressions](https://doc.rust-lang.org/reference/const_eval.html#constant-expressions).
+///
+/// The usage is almost the same as the macro [`dec_interval!(a, b)`](`dec_interval!`),
+/// except that `const_dec_interval!(a, b)` returns a [`DecInterval`]
+/// or results in a compilation error if the construction is invalid.
 #[macro_export]
 macro_rules! const_dec_interval {
     ($a:expr, $b:expr) => {{
